!*****************************************************************!
!*                                                               *!
!*              4C (FORESEE) Simulation Model                    *!
!*                                                               *!
!*                    Subroutines for:                           *!
!*                   - windows shell -                           *!
!*                                                               *!
!*  contains:                                                    *!
!*   InitMain                                                    *!
!*   InitDefaultDir                                              *!
!*   InitCtr                                                     *!
!*   InitFiles                                                   *!
!*   InitFlags                                                   *!
!*   InitNames                                                   *!
!*                                                               *!
!*   InitOut                                                     *!
!*   InitYearlyOut                                               *!
!*   InitDailyOut                                                *!
!*   InitCohyOut                                                 *!
!*   InitCohdOut                                                 *!
!*                                                               *!
!*****************************************************************!
!*
!*  Implementation and Revisions:
!*  -----------------------------
!
!  17.05.05 Su  InitFiles: Button mes initial.
!  21.11.04 ag  main dialog added
!  03.11.04 ag "default directory" dialog added: InitDefaultDir
!  31.08.04 ag  new button by Ctr: RunNr apply
!  17.08.04 ag	only init functions left, initflags added
!  17.08.04 Su	SetModelSpec: Fehler bei flag_cond (charflag)
!
!*****************************************************************!


SUBROUTINE InitMain(dlgParent)

use dflogm
use data_simul

type (dialog) dlg
type (dialog) dlgParent
type (dialog) local_dlg

external Start, Cancel

include '4C_dialogs.fd'


logical retlog
character(150) dir
integer nflag
character(90), dimension(5) :: charflag
character(20) text, helpsim

	local_dlg = dlgParent

	if ( .not. DlgInit( idd_4C_main, dlg ) ) then
		write (*,*) "error: resource not found"
	else

	 	! set functionality of the OK and CANCEL button
		retlog = DlgSetSub( dlg, IDSTOP, Cancel )
		retlog = DlgSetSub( dlg, ID_START_4C, Start )
!		retlog = DlgSetSub( dlg, IDCANCEL, Cancel )

!	    retlog = DlgSet( dlg, IDC_STATIC_4C, 3) 
!		retlog = DlgSet( dlg, IDC_STATIC_4C, 'Forest Growth Model 4C',1 )
!		retlog = DlgSet( dlg, IDC_STATIC_4C, 'PIK',2 )
!		retlog = DlgSet( dlg, IDC_STATIC_4C, '2005',3 )
	endif 	
	
	retint = DlgModal(dlg)

END SUBROUTINE InitMain


!!*************************************************************************
!SUBROUTINE InitDefaultDir(dlg)
!
!use dflogm
!
!!character*(*),parameter :: filter_spec = &
!!  "Simulation control Files (*.sim)"C//"*.sim"C// &
!!  "All Files (*.*)"C//"*"C//""C
!
!type (dialog) dlg
!
!external Forward, Cancel
!
!include '4C_dialogs.fd'
!
!
!logical retlog
!character(150) dir
! 
!    retlog = DlgInit( idd_4C_default_dir, dlg )
!	
!    call Act_Dir(dir)
!    retlog = DlgSet( dlg, IDC_EDIT_DIR_OUT, trim(dir)//'\output\')
!    retlog = DlgSet( dlg, IDC_EDIT_DIR_IN, trim(dir)//'\input\') 
! 
!
!	! set functionality of the OK and CANCEL button
!	retlog = DlgSetSub( dlg, ID_DEFAULT_DIR_BUTTON_OK,	  Forward)
!	retlog = DlgSetSub( dlg, IDCANCEL, Cancel )
!	
!	
!END SUBROUTINE InitDefaultDir
!
!!***************************************************************

!SUBROUTINE InitCtr(dlg)
!
!
!use dflogm
!
!type (dialog) dlg
!
!external Forward, Cancel
!
!include '4C_dialogs.fd'
!
!
!logical retlog
!
!    retlog = DlgInit( idd_4C_ctr, dlg )
!	! set functionality of the OK button
!	retlog = DlgSetSub( dlg, ID_CTR_BUTTON_RUNNR, Forward)
!	retlog = DlgSetSub( dlg, ID_CTR_BUTTON_OK,	  Forward)
!	retlog = DlgSetSub( dlg, ID_CTR_BUTTON_IDS, Forward)
!	retlog = DlgSetSub( dlg, ID_CTR_BUTTON_FLAGS, Forward)
!	retlog = DlgSetSub( dlg, ID_CTR_BUTTON_FILES, Forward)
!	retlog = DlgSetSub( dlg, IDCANCEL, Cancel )
!	
!	
! ! set up run option
!	retlog = DlgSet( dlg, IDC_COMBO_runv, 7) 
!!	retlog = DlgSet( dlg, IDC_COMBO_runv, '0 - single run',1) 
!	retlog = DlgSet( dlg, IDC_COMBO_runv, '1 - standard run',1) 
!	retlog = DlgSet( dlg, IDC_COMBO_runv, '2 - climate sensitivity',2) 
!	retlog = DlgSet( dlg, IDC_COMBO_runv, '3 - multi run with parameter choice',3) 
!	retlog = DlgSet( dlg, IDC_COMBO_runv, '4 - multi run with different data sets',4) 
!	retlog = DlgSet( dlg, IDC_COMBO_runv, '5 - multi run with SPRINT/SIMENV, only on SP',5) 
!	retlog = DlgSet( dlg, IDC_COMBO_runv, '6 - multi run with the same intialization for every year',6) 
!	retlog = DlgSet( dlg, IDC_COMBO_runv, '7 - multi run for one site description and a list of control files',7) 
!
!
!
!END SUBROUTINE InitCtr
!
!!***************************************************************
!
!SUBROUTINE InitFiles(dlg)
!
!use dflogm
!USE DFLIB
!
!use data_simul
!
!implicit none
!
!type (dialog) dlg
!integer id
!integer callbacktype
!integer retint
!logical ex
!character($MAXPATH) dir
!
!
!include '4C_dialogs.fd'
!
!logical retlog
!external BrowseFile, Forward, Cancel 
!
!    retlog = DlgInit( idd_4C_files, dlg )
!
!	retlog = DlgSetSub( dlg, IDC_BUTTON_cli, BrowseFile )
!	retlog = DlgSetSub( dlg, IDC_BUTTON_specpar, BrowseFile )
!	retlog = DlgSetSub( dlg, IDC_BUTTON_sop, BrowseFile )
!	retlog = DlgSetSub( dlg, IDC_BUTTON_soi, BrowseFile )
!	retlog = DlgSetSub( dlg, IDC_BUTTON_sop, BrowseFile )
!	retlog = DlgSetSub( dlg, IDC_BUTTON_man, BrowseFile )
!	retlog = DlgSetSub( dlg, IDC_BUTTON_dep, BrowseFile )
!	retlog = DlgSetSub( dlg, IDC_BUTTON_red, BrowseFile )
!	retlog = DlgSetSub( dlg, IDC_BUTTON_lit, BrowseFile )
!	retlog = DlgSetSub( dlg, IDC_BUTTON_wpm, BrowseFile )
!	retlog = DlgSetSub( dlg, IDC_BUTTON_mes, BrowseFile )
!	retlog = DlgSetSub( dlg, IDC_BUTTON_ini, BrowseFile )
!
!
!	retlog = DlgSetSub( dlg, ID_FILES_BUTTON_OK, Forward )	
!!	retlog = DlgSetSub( dlg, IDCANCEL, Cancel )	
!	retlog = DlgSetSub( dlg, ID_CANCEL_FILES, Cancel )	
!
!END SUBROUTINE InitFiles
!
!!***************************************************************
!
!SUBROUTINE InitFlags(dlg)
!
!use dflogm
!USE DFLIB
!
!use data_simul
!
!implicit none
!
!type (dialog) dlg
!integer id
!integer callbacktype
!integer retint
!logical ex
!character($MAXPATH) dir
!
!
!include '4C_dialogs.fd'
!
!logical retlog
!external Forward, Cancel
!
!    retlog = DlgInit( idd_4C_flags, dlg )
!
!	retlog = DlgSetSub( dlg, ID_FLAGS_BUTTON_OK, Forward )	
!	retlog = DlgSetSub( dlg, IDCANCEL, Cancel )	
!
!END SUBROUTINE InitFlags
!
!!***************************************************************
!
!SUBROUTINE InitIds(dlg)
!
!use dflogm
!USE DFLIB
!
!use data_simul
!
!implicit none
!
!type (dialog) dlg
!include '4C_dialogs.fd'
!
!logical retlog
!external Forward, Cancel
!
!    retlog = DlgInit( idd_4C_ids, dlg )
!
!	retlog = DlgSetSub( dlg, ID_IDS_BUTTON_OK, Forward )	
!!	retlog = DlgSetSub( dlg, IDCANCEL, Cancel )	
!	retlog = DlgSetSub( dlg, ID_CANCEL_FLAGS, Cancel )	
!
!END SUBROUTINE 
!
!
!!***************************************************************
!
!recursive SUBROUTINE InitOut(dlg)
!
!use dflogm
!USE DFLIB
!
!use data_out
!use data_simul
!
!implicit none
!
!type (dialog) dlg
!integer id
!integer callbacktype
!integer retint, num
!logical check_state
!character($MAXPATH) dir
!character(20) text, helpsim
!
!external Cancel, Forward, StepBack
!
!include '4C_dialogs.fd'
!
!logical retlog
!
!    retlog = DlgInit( idd_4C_out, dlg )
!
!	! buttons' controls 
!	retlog = DlgSetSub( dlg, IDC_BUTTON_yearly, Forward )
!	retlog = DlgSetSub( dlg, IDC_BUTTON_daily, Forward )
!	retlog = DlgSetSub( dlg, IDC_BUTTON_coh_yearly, Forward )
!	retlog = DlgSetSub( dlg, IDC_BUTTON_coh_daily, Forward )
!
! ! summation output
!	retlog = DlgSet( dlg, IDC_COMBO_sum, 5) 
!	retlog = DlgSet( dlg, IDC_COMBO_sum, ' 0 - no output',1) 
!	retlog = DlgSet( dlg, IDC_COMBO_sum, ' 1 - daily output',2) 
!	retlog = DlgSet( dlg, IDC_COMBO_sum, ' 2 - output with time step of photosynthesis',3) 
!	retlog = DlgSet( dlg, IDC_COMBO_sum, ' 3 - monthly output',4) 
!	retlog = DlgSet( dlg, IDC_COMBO_sum, ' 4 - yearly output',5) 
!
!	retlog = DlgSetSub( dlg, ID_START, Forward )	
!	retlog = DlgSetSub( dlg, ID_SAVE, Forward )	
!	retlog = DlgSetSub( dlg, ID_OUT_BUTTON_BACK, StepBack )
!	retlog = DlgSetSub( dlg, IDCANCEL, Cancel )	
!!	retlog = DlgSetSub( dlg, ID_CANCEL_OUT, Cancel )	
!
!END SUBROUTINE InitOut
!
!
!!*************************************************************
!
!SUBROUTINE InitYearlyOut( dlg )
!
!use dflogm
!USE DFLIB
!
!use data_out
!use data_simul
!
!implicit none
!
!type (dialog) dlg
!integer retint, num, i, nflag  
!!logical check_state
!character(90), dimension(5) :: charflag
!
!external Cancel, Forward, EditYearlyOut
!
!include '4C_dialogs.fd'
!
!logical retlog
!
!    retlog = DlgInit( idd_4C_yearly, dlg )
!
!	retlog = DlgSetSub( dlg, IDCANCEL, Cancel )	
!	retlog = DlgSetSub( dlg, ID_YEARLY_BUTTON_OK, Forward )	
!	retlog = DlgSetSub( dlg, ID_YEARLY_BUTTON_SELECT, Forward )	
!	retlog = DlgSetSub( dlg, ID_YEARLY_BUTTON_DESELECT, Forward )	
!!	retlog = DlgSetSub( dlg, ID_YEARLY_BUTTON_SELECT, EditYearlyOut )	
!    
!END SUBROUTINE InitYearlyOut
!
!
!!***************************************************************
!
!SUBROUTINE InitDailyOut( dlg )
!
!use dflogm
!USE DFLIB
!
!use data_out
!use data_simul
!
!implicit none
!
!type (dialog) dlg
!external Cancel, Forward
!
!include '4C_dialogs.fd'
!
!logical retlog
!
!    retlog = DlgInit( idd_4C_daily, dlg )
!
!
!!	retlog = DlgSetSub( dlg, IDCANCEL, Cancel )	
!	retlog = DlgSetSub( dlg, ID_CANCEL_OUTF, Cancel )	
!	retlog = DlgSetSub( dlg, ID_DAILY_BUTTON_OK, Forward )	
!	retlog = DlgSetSub( dlg, ID_DAILY_BUTTON_SELECT, Forward )	
!	retlog = DlgSetSub( dlg, ID_DAILY_BUTTON_DESELECT, Forward )	
!
!END SUBROUTINE InitDailyOut
!
!
!!***************************************************************
!
!SUBROUTINE InitCohdOut( dlg )
!
!use dflogm
!USE DFLIB
!
!use data_out
!use data_simul
!
!implicit none
!
!type (dialog) dlg
!integer retint, num
!logical check_state
!!character($MAXPATH) dir
!character(20) text, helpsim
!
!external Cancel, Forward
!
!include '4C_dialogs.fd'
!
!logical retlog
!
!    retlog = DlgInit( idd_4C_coh_daily, dlg )
!
!
!	retlog = DlgSetSub( dlg, IDCANCEL, Cancel )	
!	retlog = DlgSetSub( dlg, ID_DAILYCOH_BUTTON_OK, Forward )	
!	retlog = DlgSetSub( dlg, ID_DAILYCOH_BUTTON_SELECT, Forward )	
!	retlog = DlgSetSub( dlg, ID_DAILYCOH_BUTTON_DESELECT, Forward )	
!
!END SUBROUTINE InitCohdOut
!
!
!!***************************************************************
!SUBROUTINE InitCohyOut( dlg )
!
!use dflogm
!USE DFLIB
!
!use data_out
!use data_simul
!
!implicit none
!
!type (dialog) dlg
!integer retint, num
!logical check_state
!character($MAXPATH) dir
!character(20) text, helpsim
!
!external Cancel, Forward
!
!include '4C_dialogs.fd'
!
!logical retlog
!
!    retlog = DlgInit( idd_4C_coh_yearly, dlg )
!
!
!	retlog = DlgSetSub( dlg, IDCANCEL, Cancel )	
!	retlog = DlgSetSub( dlg, ID_YEARLYCOH_BUTTON_OK, Forward )	
!	retlog = DlgSetSub( dlg, ID_YEARLYCOH_BUTTON_SELECT, Forward )	
!	retlog = DlgSetSub( dlg, ID_YEARLYCOH_BUTTON_DESELECT, Forward )	
!
!END SUBROUTINE InitCohyOut


