!*****************************************************************!
!*                                                               *!
!*              4C (FORESEE) Simulation Model                    *!
!*                                                               *!
!*                    Subroutines for:                           *!
!*                                                               *!
!*    - data_general_dlg: module to store the general dlg info	 *!
!*    - flag_field :      module to store the flags				 *!
!*    - flags_comments:   module to store the flags commens		 *!
!*                                                               *!                 
!*	  - FlagsToStructure: stores the flags to the simulation     *!
!*                        structure                              *!
!*    - StructureToFlags: stores the read structure to the flags *!                 
!*                        field                                  *!                 
!*                                                               *!    
!*    - flags_ini:		  initialize the values of the modules   *!                 
!*                                                               *!                              
!*                                                               *!      
!*****************************************************************!
!*
!*  Implementation and Revisions:
!*  -----------------------------
!
! Latest changes:
!
!  date     who  what
!  -------------------------------------------------------------
!
!  17.05.16  Su  FlagsToStructure: flag_growth ersetzt durch flag_dis
!  18.08.11  Su  flags_ini: flag_wurz, flag_stat erweitert
!  15.08.11 Su   flags_comments: flag_CO2_com verlngert
!  11.08.11  Su  flags_ini: flag_CO2 erweitert
!  27.07.11  Su  flags_ini: flag_decomp, flag_limi, flag_mg erweitert
!  08.08.05  Su  flags_ini: flag_reg erweitert
!  27.07.05  Su  flags_ini: flag_limi erweitert
!  15.07.05  Su  flags_ini: flag_limi erweitert
!  17.05.05  Su  flags_comments, flags_ini, FlagsToStructure, StructureToFlags: flag_stat
!  23.02.05  ag  flags dimension to 30, flag_reg set for FORGRA
!  15.11.04  ag  module for dlg general information
!  14.09.04  ag  FlagsToStructure, StructureToFlags
!  09.09.04  ag  subroutine added
!  02.09.04  ag  installed
!


!***********************************************************************
module data_general_dlg
	
	! which way was chosen
	integer :: id_ctr	

end module

!***********************************************************************


! data module containig the flags changes for the win_appl
module flag_field

	integer,allocatable,save,dimension(:,:)	::	flagsave
	character(50), dimension(22) :: flagnames	
	integer act_run

end ! module flags_field

!contains all flag comments for the combo-boxes
module flags_comments
	character(60), dimension(30) :: flag_mort_com
	character(60), dimension(30) :: flag_reg_com
	character(60), dimension(30) :: flag_forska_com
	character(60), dimension(30) :: flag_stand_com
	character(60), dimension(30) :: flag_sveg_com
	character(60), dimension(30) :: flag_mg_com
	character(60), dimension(30) :: flag_dis_com
	character(60), dimension(30) :: flag_light_com
	character(60), dimension(30) :: flag_folhei_com
	character(60), dimension(30) :: flag_volfunc_com
	character(60), dimension(30) :: flag_resp_com
	character(60), dimension(30) :: flag_limi_com
	character(60), dimension(30) :: flag_decomp_com
	character(60), dimension(30) :: flag_sign_com
	character(60), dimension(30) :: flag_wred_com
	character(60), dimension(30) :: flag_wurz_com
	character(60), dimension(30) :: flag_cond_com
	character(60), dimension(30) :: flag_int_com
	character(60), dimension(30) :: flag_eva_com
	character(100), dimension(30) :: flag_CO2_com
	character(60), dimension(30) :: flag_sort_com
	character(60), dimension(30) :: flag_wpm_com
	character(60), dimension(30) :: flag_stat_com
end

!!***************************************************************
!! store flags from the read structure
!SUBROUTINE StructureToFlags()
!
!use dflogm
!USE DFLIB
!
!use data_out
!use data_simul
!use data_tsort
!use flag_field
!
!implicit none
!
!character (100) simpara
!integer i, j, run
!	
!	
!	! store first flag set for all runs
!	DO i=1,site_nr
!		! store all flags values
!		flagsave(:,i) = (/ flag_mort, flag_reg, flag_forska, flag_stand, flag_sveg,		&
!						flag_mg, flag_dis, flag_light, flag_folhei, flag_volfunc,		&
!						flag_resp, flag_limi, flag_decomp, flag_sign, flag_wred,		&
!						flag_wurz, flag_cond, flag_int, flag_eva, flag_co2, flag_sort,	&
!						flag_wpm, flag_stat /)
!	end do
!	
!	
!	! store flags changes for following runs
!	if(site_nr .gt. 1) then  
!		run = 1		
!		DO j=1,100
!			if (vpar(j) .gt. -99.0) then
!                simpara = simpar(j)
!             	run = run + 1
!				DO i=1, size(flagnames) 
!					if (trim(simpara) .eq. flagnames(i)) then
!						flagsave(i, run)=vpar(j)
!						return
!					endif
!				end do
!			end if
!		end do
!	end if
!
!	
!END SUBROUTINE StructureToFlags
!
!!***************************************************************
!
!! store flags to the read structure
!SUBROUTINE FlagsToStructure()
!
!use dflogm
!USE DFLIB
!
!use data_out
!use data_simul
!use data_tsort
!use flag_field
!
!implicit none
!
!character (100) simpara
!integer i, j, jsim
!	
!	
!	! store first flag set for the first run
!	flag_mort	= flagsave(1,1) 
!	flag_reg	= flagsave(2,1)
!	flag_forska = flagsave(3,1)
!	flag_stand	= flagsave(4,1)
!	flag_sveg	= flagsave(5,1)
!	flag_mg		= flagsave(6,1)
!	flag_dis	= flagsave(7,1)
!	flag_light	= flagsave(8,1)
!	flag_folhei	= flagsave(9,1)
!	flag_volfunc = flagsave(10,1)
!	flag_resp	= flagsave(11,1)
!	flag_limi	= flagsave(12,1)
!	flag_decomp	= flagsave(13,1)
!	flag_sign	= flagsave(14,1)
!	flag_wred	= flagsave(15,1)
!	flag_wurz	= flagsave(16,1)
!	flag_cond	= flagsave(17,1)
!	flag_int	= flagsave(18,1)
!	flag_eva	= flagsave(19,1)
!	flag_co2	= flagsave(20,1)
!	flag_sort	= flagsave(21,1)
!	flag_wpm    = flagsave(22,1)
!	flag_stat   = flagsave(23,1)
!	
!	! store flags changes for following runs
!	if(site_nr .gt. 1) then  
!		jsim = 1	
!		DO i=2, site_nr 
!			DO j=1, size(flagnames)
!				if ( flagsave(j,i-1) /= flagsave(j,i) ) then
!					simpar(jsim) = flagnames(j)
!					vpar(jsim)	 = flagsave(j,i)
!					jsim = jsim + 1
!				end if
!			END DO
!			vpar(jsim) = -99.0
!			jsim = jsim + 1
!		END DO
!	end if
!	
!END SUBROUTINE FlagsToStructure
!
!!**************************************************************************
!! fills the flags' names and comments
!subroutine flags_ini()
!
!use flag_field
!use flags_comments
!
!implicit none
!
!	! the number of actual run
!	act_run = 1	
!	! list of the flag names	
!	flagnames(1)  = 'flag_mort'
!	flagnames(2)  = 'flag_reg'
!	flagnames(3)  = 'flag_forska'
!	flagnames(4)  = 'flag_stand'
!	flagnames(5)  = 'flag_sveg'
!	flagnames(6)  = 'flag_mg'
!	flagnames(7)  = 'flag_dis'
!	flagnames(8)  = 'flag_light'
!	flagnames(9)  = 'flag_folhei'
!	flagnames(10) = 'flag_volfunc'
!	flagnames(11) = 'flag_resp'
!	flagnames(12) = 'flag_limi'
!	flagnames(13) = 'flag_decomp'
!	flagnames(14) = 'flag_sign'
!	flagnames(15) = 'flag_wred'
!	flagnames(16) = 'flag_wurz'
!	flagnames(17) = 'flag_cond'
!	flagnames(18) = 'flag_int'
!	flagnames(19) = 'flag_eva'
!	flagnames(20) = 'flag_CO2'
!    flagnames(21) = 'flag_sort'
!	flagnames(22) = 'flag_wpm'
!
!!flag_mort
!flag_mort_com(1) = '0 -  without mortality'
!flag_mort_com(2) = '1 -  with stress mortality only'
!flag_mort_com(3) = '2 -  with stress and intrinsic mortality'
!!do i = nflag+1, ichar
! !  flag_mort_com(i) = ' '
!!enddo
!
!
!!flag_reg
!flag_reg_com(1) = ' 0 -  without regeneration'
!flag_reg_com(2) = ' 1 -  with regeneration'
!flag_reg_com(3) = ' 2 -  with regeneration and weekly seedling growth'
!flag_reg_com(4) = ' 3 -  generation of several cohorts at one time step'
!flag_reg_com(5) = '10 -  planting of pine seedlings'
!flag_reg_com(6) = '11 -  planting of beech seedlings'
!flag_reg_com(7) = '12 -  planting of oak seedlings'
!flag_reg_com(8) = '13 -  planting of spruce seedlings'
!flag_reg_com(9) = '14 -  planting of birch seedlings'
!flag_reg_com(10)= '15 -  planting of aspen seedlings'
!flag_reg_com(11)= '20 -  planting of small trees in the first year given by data from file *.pla'
!flag_reg_com(12)= '30 -  regeneration FORGRA'
!
!!flag_forska
!flag_forska_com(1) = ' 0 '
!
!!flag_stand
!flag_stand_com(1) = '0 -  without initialisation '
!flag_stand_com(2) = '1 -  read data from File *.ini'
!flag_stand_com(3) = '2 -  generate new *.ini from original data'
!
!!flag_sveg
!flag_sveg_com(1) = '0 -  no ground vegetation '
!flag_sveg_com(2) = '1 -  initialisation of ground vegetation with fixed data set'
!
!!flag_mg
!flag_mg_com(1) = '0  -  without management '
!flag_mg_com(2) = '1  -  simple management routine'
!flag_mg_com(3) = '2  -  new management routines (thinnings)'
!flag_mg_com(4) = '3  -  target based management, using target stem numbers for thinning (adapted for OakChain)'
!flag_mg_com(5) = '4  -  seeding routine, seeds once in simulation time'
!flag_mg_com(6) = '5  -  thinning of dead trees, which are removed from stand (no litter input)'
!flag_mg_com(7) = '9  -  Austrian management'
!flag_mg_com(8) = '10 -  disturbance management (Bavaria)'
!flag_mg_com(9) = '33 -  target based management (using relative biomass reduction parameter)'
!
!!flag_dis
!flag_dis_com(1) = '0 - without disturbance '
!
!!flag_light
!flag_light_com(1) = '1 -  classical gap model approach'
!flag_light_com(2) = '2 -  cohorts with crown projection area, light is averaged after each layer'
!flag_light_com(3) = '3 -  cohorts with crown projection area, light is absorbed per layer and cohort'
!flag_light_com(4) = '4 -  cohorts with crown projection area, light is absorbed per layer and cohort and modified by average yearly sun inclination'
!
!!flag_folhei
!flag_folhei_com(1) = '0 -  old (linear)'
!flag_folhei_com(2) = '1 -  new (nonlinear)'
!
!!flag_volfunc
!flag_volfunc_com(1) = '0 -  old (two sapwood cones) '
!flag_volfunc_com(2) = '1 -  new (sapwood cylinder below crown base and cone above) '
!
!!flag_resp
!flag_resp_com(1) = '0 -  tree respiration equals 0.52*NPP  '
!flag_resp_com(2) = '1 -  tree respiration modelled with organ specific respiration rates  '
!
!!flag_limi
!flag_limi_com(1)  = '0  -  no limitation   '
!flag_limi_com(2)  = '1  -  only drought limitation '
!flag_limi_com(3)  = '2  -  only nitrogen limitation with fixed RedN'
!flag_limi_com(4)  = '3  -  drought and nitrogen limitation with fixed RedN'
!flag_limi_com(5)  = '4  -  nitrogen dynamic from leaf N, only nitrogen limitation'
!flag_limi_com(6)  = '5  -  nitrogen dynamic from leaf N, drought and nitrogen limitation'
!flag_limi_com(7)  = '6  -  nitrogen dynamic from mineralisation, only nitrogen limitation'
!flag_limi_com(8)  = '7  -  nitrogen dynamic from mineralisation, drought and nitrogen limitation'
!flag_limi_com(9)  = '8  -  only nitrogen limitation, age-dependent'
!flag_limi_com(10) = '9  -  drought and age-dependent nitrogen limitation'
!flag_limi_com(11) = '10 -  drought and nitrogen limitation; below 120 kg N/ha decrease depending on N demand and N mineralisation'
!
!!flag_decomp
!flag_decomp_com(1) = '0  -  uniform decomposition model of all litter fractions (incl. stem wood and coarse roots) '
!flag_decomp_com(2) = '1  -  stem wood and coarse root decay in a separate model with no direct N mineralisation and input into the humus layer according to N release '
!flag_decomp_com(3) = '10 -  like 0; N uptake distribution controlled by N-supply of cohorts '
!flag_decomp_com(4) = '11 -  like 1; N uptake distribution controlled by N-supply of cohorts '
!flag_decomp_com(5) = '20 -  like 0; read litter input '
!flag_decomp_com(6) = '21 -  like 1; read litter input '
!flag_decomp_com(7) = '30 -  like 0; no litter input '
!flag_decomp_com(8) = '31 -  like 1; no litter input '
!flag_decomp_com(9) = '40 -  ike 0; N uptake controlled by C/N-ratio of non-heartwood '
!flag_decomp_com(10)= '41 -  like 1; N uptake controlled by C/N-ratio of non-heartwood '
!
!!flag_sign
!flag_sign_com(1) = '0 -  constant sigman  '
!flag_sign_com(2) = '1 -  sigman varies with tree dimension  '
!
!!flag_wred 
!flag_wred_com(1) = '1 -  function with uptake inhibition near wilting point and field capcity'
!flag_wred_com(2) = '2 -  uptake restricted to 4%  '
!flag_wred_com(3) = '3 -  reduction along a profile '
!flag_wred_com(4) = '4 -  specific reduction function for Beerenbusch'
!flag_wred_com(5) = '5 -  no reduction '
!
!!flag_wurz
!flag_wurz_com(1) = '0 -  equally distributed over all layers'
!flag_wurz_com(2) = '1 -  root profile'
!flag_wurz_com(3) = '2 -  input of root profile after request; unchanged during the simulation time'
!flag_wurz_com(4) = '3 -  root profile: Arora approach'
!flag_wurz_com(5) = '4 -  root profile: TRAP model'
!flag_wurz_com(6) = '5 -  Jackson approach with root depth following Arora'
!flag_wurz_com(7) = '6 -  Jackson approach with root depth depending on water shortage'
!
!!flag_cond
!flag_cond_com(1) = ' 0 -  thermal conductivity and capacity: de Vries-approach; soil surface temperature equals temperature of 1. layer '
!flag_cond_com(2) = ' 1 -  thermal conductivity and capacity: Neusypina-approach; soil surface temperature equals temperature of 1. layer '
!flag_cond_com(3) = '10 -  thermal conductivity and capacity: de Vries-approach; real soil surface temperature (additional to the soil profile)'
!flag_cond_com(4) = '11 -  thermal conductivity and capacity: Neusypina-approach; real soil surface temperature (additional to the soil profile)'
!flag_cond_com(5) = '20 -  thermal conductivity and capacity: de Vries-approach; reading soil surface temperature after request'
!flag_cond_com(6) = '21 -  thermal conductivity and capacity:  Neusypina-approach; reading soil surface temperature after request'
!
!!flag_int
!flag_int_com(1) = '0 -  total interception of canopy and ground vegetation (with total crown storage capacity) '
!flag_int_com(2) = '1 -  interception for each cohort (with storage capacity in each crown layer and with distribution of precipitation over all canopy layers)'
!flag_int_com(3) = '2 -  interception for each cohort (with storage capacity for each cohort); precipitation distributed according to foliage'
!flag_int_com(4) = '3 -  pine interception from Anders (storage capacity = 2.9 mm)'
!flag_int_com(5) = '4 -  fit for Level II Brandenburg, pine (20% of precipitation)'
!flag_int_com(6) = '5 -  30% of precipitation (for spruce); faster evaporation'
!
!!flag_eva
!flag_eva_com(1) = '0 -  Turc / Ivanov '
!flag_eva_com(2) = '1 -  Priestley / Taylor'
!flag_eva_com(3) = '2 -  Priestley / Taylor for each cohort'
!flag_eva_com(4) = '3 -  Penman / Monteith'
!flag_eva_com(5) = '4 -  Penman / Monteith for each cohort'
!flag_eva_com(6) = '5 -  Haude'
!
!!flag_CO2
!flag_CO2_com(1)  = '0   -  constant atmospheric mixing ratio as specified in model.par  '
!flag_CO2_com(2)  = '101 -  Kohlmaier function: increasing atmospheric mixing ratio as historic increase '
!flag_CO2_com(3)  = '102 -  LTEEF scenario: increasing atmospheric mixing ratio as historic increase '
!flag_CO2_com(4)  = '103 -  Mauna Loa series: increasing atmospheric mixing ratio as historic increase '
!flag_CO2_com(5)  = '110 -  IPCC IS92a after Bern CC model: increasing atmospheric mixing ratio as historic increase '
!flag_CO2_com(6)  = '111 -  IPCC SRES A1FI after Bern CC mode: increasing atmospheric mixing ratio as historic increase '
!flag_CO2_com(7)  = '112 -  IPCC SRES A2 after Bern CC mode: increasing atmospheric mixing ratio as historic increase '
!flag_CO2_com(8)  = '113 -  IPCC SRES B1 after Bern CC model: increasing atmospheric mixing ratio as historic increase '
!flag_CO2_com(9)  = '114 -  IPCC B2 after Bern CC model: increasing atmospheric mixing ratio as historic increase '
!flag_CO2_com(10) = '115 -  IPCC A1B after Bern CC model: increasing atmospheric mixing ratio as historic increase '
!flag_CO2_com(11) = '116 -  IPCC A1p after Bern CC model: increasing atmospheric mixing ratio as historic increase '
!flag_CO2_com(12) = '201 -  Kohlmaier function: step change in the middle of the simulation period '
!flag_CO2_com(13) = '202 -  LTEEF scenario: step change in the middle of the simulation period '
!flag_CO2_com(14) = '203 -  Mauna Loa series: step change in the middle of the simulation period '
!flag_CO2_com(15) = '210 -  IPCC IS92a after Bern CC model: step change in the middle of the simulation period '
!flag_CO2_com(16) = '211 -  IPCC SRES A1FI after Bern CC mode: step change in the middle of the simulation period '
!flag_CO2_com(17) = '212 -  IPCC SRES A2 after Bern CC mode: step change in the middle of the simulation period '
!flag_CO2_com(18) = '213 -  IPCC SRES B1 after Bern CC model: step change in the middle of the simulation period '
!flag_CO2_com(19) = '214 -  IPCC B2 after Bern CC model: step change in the middle of the simulation period '
!flag_CO2_com(20) = '215 -  IPCC A1B after Bern CC model: step change in the middle of the simulation period '
!flag_CO2_com(21) = '216 -  IPCC A1p after Bern CC model: step change in the middle of the simulation period '
!flag_CO2_com(22) = 'xxx -  setting of constant atmospheric mixing ratio yyy > 300 ppm (integer)  '
!
!!flag_sort
!flag_sort_com(1) = '0 -  all assortments '
!flag_sort_com(2) = '1 -  without stems (Stammholz)'
!flag_sort_com(3) = '2 -  only LAS with  3m + industrial + fuel wood'
!flag_sort_com(4) = '3 -  only LAS with  4m + industrial + fuel wood'
!
!!flag_wpm
!flag_wpm_com(1) = '0 -  no wpm calculation '
!flag_wpm_com(2) = '1 -  wood product model calculation'
!flag_wpm_com(3) = '2 -  sea calculation'
!flag_wpm_com(4) = '3 -  sea and wpm'
!flag_wpm_com(5) = '11 -  wpm with nothern europe parameters'
!flag_wpm_com(6) = '21 -  wpm with southern europe parameters'
!
!!flag_stat
!flag_stat_com(1) = '0 -  no analysis '
!flag_stat_com(2) = '1 -  comparison with measurements '
!flag_stat_com(3) = '2 -  additional to 1 output of the  tripels of measurement , simulation value and residual '
!
!
!
!end subroutine flags_ini
