!*****************************************************************!
!*                                                               *!
!*              4C (FORESEE) Simulation Model                    *!
!*                                                               *!
!*                    Subroutines for:                           *!
!*                   - windows shell -                           *!
!*                                                               *!
!*  contains:                                                    *!
!*   BrowseFile                                                  *!
!*   GetFile                                                     *!
!*   GetFileF                                                    *!
!*   SetFlag                                                     *!
!*   SetFlag100                                                  *!
!*   SetCheckOut                                                 *!
!*                                                               *!
!*****************************************************************!
!*
!*  Implementation and Revisions:
!*  -----------------------------
!
!  07.01.16 Su GetFile: Lnge filename angepasst f. FileOpen
!              SetCheckOut: outa mit dimension outy_n anlegen
!  28.10.11 Su SetCheckOut: Dimension von outa erweitert
!  15.08.11 Su SetFlag100 neu
!              GetFlag: Textlaenge dynamisch
!  11.08.11 Su SetFlag: Dimension charflag auf 30 erweitert
!  17.05.05 Su BrowseFile: mesfile(1) festlegen
!                          actDir nach GetFile reaktivieren
!  01.02.05 ag BrowseFile: after reading the new standid file, edit the standid combo box
!  25.11.04 ag Act_Dir_old deleted
!  09.09.04 ag SetFlag changed for flag field use
!  17.08.04 ag set flag: number of arguments changed, callbacktype deleted 
!  17.08.04 Su 


!SUBROUTINE BrowseFile ( dlg, id, callbacktype )
!
!use dflogm
!use dflib
!
!use data_mess
!use data_simul
!use flag_field
!
!implicit none
!type (dialog) dlg
!integer id
!integer callbacktype
!integer retint
!!logical ex
!character(512) :: filename
!character(150) :: text 
!
!!Settings for FileOpen
!character*(512):: filter_spec
!!character*(*),parameter :: filter_spec = &
!!  "Climate data files (*.cli)"C//"*.cli"C// &
!!  "All Files (*.*)"C//"*.*"C//""C
!
!include '4C_dialogs.fd'
!
!logical retlog
!Logical retdir
!
!    ! supress compiler warnings for unreferenced arguments
!	integer local_id, local_callbacktype
!	local_id = id
!	local_callbacktype = callbacktype
!	
!
!	filename = ' '
!
!	select case (id)
!	  case (IDC_BUTTON_cli)
!        filter_spec = "Climate data files (*.cli)"C//"*.cli"C// &
!                      "All Files (*.*)"C//"*.*"C//""C
!        text = ' Climate data file: '
!		print *, 'filename ' , filename
!        call GetFile (filename, filter_spec, text )
!
!		if ( filename .eq. ' ') then       
!			retlog = DlgGet(dlg, IDC_EDIT_cli, climfile(act_run))
!		else
!			climfile(act_run) = filename
!			retlog = DlgSet( dlg, IDC_EDIT_cli, climfile(act_run)) 
!		endif
!		
!
!	  case (IDC_BUTTON_specpar)
!        filter_spec = "Species parameter files (*.par)"C//"*.par"C// &
!                      "All Files (*.*)"C//"*.*"C//""C
!        text = ' Species parameter file: '
!        call GetFile (filename, filter_spec, text )
!		if ( filename .eq. ' ') then       
!			retlog = DlgGet(dlg, IDC_BUTTON_specpar, specfile(act_run))
!		else
!			specfile(act_run) = filename
!			retlog = DlgSet( dlg, IDC_EDIT_specpar, specfile(act_run)) 
!		endif
!
!	  case (IDC_BUTTON_sop)
!        filter_spec = "Soil parameter files (*.sop)"C//"*.sop"C// &
!                      "All Files (*.*)"C//"*.*"C//""C
!        text = ' Soil parameter file: '
!        call GetFile (filename, filter_spec, text )
!        if ( filename .eq. ' ') then       
!			retlog = DlgGet(dlg, IDC_EDIT_sop, sitefile(act_run)) 
!		else
!			sitefile(act_run) = filename
!			retlog = DlgSet( dlg, IDC_EDIT_sop, sitefile(act_run)) 
!		endif
!
!	  case (IDC_BUTTON_soi)
!        filter_spec = "Soil initialisation files (*.soi)"C//"*.soi"C// &
!                      "All Files (*.*)"C//"*.*"C//""C
!        text = ' Soil initialisation file: '
!        call GetFile (filename, filter_spec, text )
!		if ( filename .eq. ' ') then       
!			retlog = DlgGet(dlg, IDC_EDIT_soi, valfile(act_run)) 
!		else
!			valfile(act_run) = filename
!			retlog = DlgSet( dlg, IDC_EDIT_soi, valfile(act_run)) 
!		endif
!
!	  case (IDC_BUTTON_man)
!        filter_spec = "Management files (*.man)"C//"*.man"C// &
!                      "All Files (*.*)"C//"*.*"C//""C
!        text = ' Management file: '
!        call GetFile (filename, filter_spec, text )
!		if ( filename .eq. ' ') then       
!			retlog = DlgGet( dlg, IDC_EDIT_man, manfile(act_run)) 
!		else
!			manfile(act_run) = filename
!			retlog = DlgSet( dlg, IDC_EDIT_man, manfile(act_run)) 
!		endif
!
!	  case (IDC_BUTTON_dep)
!        filter_spec = "Deposition data files (*.dep)"C//"*.dep"C// &
!                      "All Files (*.*)"C//"*.*"C//""C
!        text = ' Deposition data file: '
!        call GetFile (filename, filter_spec, text )
!        if ( filename .eq. ' ') then       
!			retlog = DlgGet(dlg, IDC_EDIT_dep, depofile(act_run)) 
!		else
!			depofile(act_run) = filename
!			retlog = DlgSet( dlg, IDC_EDIT_dep, depofile(act_run)) 
!		endif
!
!	  case (IDC_BUTTON_red)
!        filter_spec = "N-reduction files (*.red)"C//"*.red"C// &
!                      "All Files (*.*)"C//"*.*"C//""C
!        text = ' N-reduction file: '
!        call GetFile (filename, filter_spec, text )
!		if ( filename .eq. ' ') then       
!			retlog = DlgGet( dlg, IDC_EDIT_red, redfile(act_run)) 
!		else
!			redfile(act_run) = filename
!			retlog = DlgSet( dlg, IDC_EDIT_red, redfile(act_run)) 
!		endif
!
!	  case (IDC_BUTTON_lit)
!        filter_spec = "Litter initialisation files (*.lit)"C//"*.lit"C// &
!                      "All Files (*.*)"C//"*.*"C//""C
!        text = ' Litter initialisation file: '
!        call GetFile (filename, filter_spec, text )
!		if ( filename .eq. ' ') then       
!			retlog = DlgGet( dlg, IDC_EDIT_lit, litfile(act_run)) 
!		else
!			litfile(act_run) = filename
!			retlog = DlgSet( dlg, IDC_EDIT_lit, litfile(act_run)) 
!		endif
!
!	  case (IDC_BUTTON_ini)
!        filter_spec = "Stand initialisation files (*.ini)"C//"*.ini"C// &
!                      "All Files (*.*)"C//"*.*"C//""C
!        text = ' Stand initialisation file: '
!        call GetFile (filename, filter_spec, text )
!		if ( filename .eq. ' ') then       
!			retlog = DlgGet(  dlg, IDC_EDIT_ini, treefile(act_run)) 
!		else
!			treefile(act_run) = filename
!			retlog = DlgSet( dlg, IDC_EDIT_ini, treefile(act_run)) 
!		endif
!		call EditStandId(dlg)
!
!	  case (IDC_BUTTON_wpm)
!        filter_spec = "Stand initialisation files (*.wpm)"C//"*.wpm"C// &
!                      "All Files (*.*)"C//"*.*"C//""C
!        text = ' WPM spinup file: '
!        call GetFile (filename, filter_spec, text )
!		if ( filename .eq. ' ') then       
!			retlog = DlgGet(  dlg, IDC_EDIT_wpm, wpmfile(act_run)) 
!		else
!			wpmfile(act_run) = filename
!			retlog = DlgSet( dlg, IDC_EDIT_wpm, wpmfile(act_run)) 
!		endif
!
!	  case (IDC_BUTTON_mes)
!        filter_spec = "Measurement files (*.mes)"C//"*.mes"C// &
!                      "All Files (*.*)"C//"*.*"C//""C
!        text = ' File with measurements: '
!        call GetFile (filename, filter_spec, text )
!	    if (not(allocated(mesfile))) allocate (mesfile(anz_mesf))
!        if ( filename .eq. ' ') then       
!			retlog = DlgGet(  dlg, IDC_EDIT_mes, mesfile(1)) 
!		else
!			mesfile(1) = filename
!			retlog = DlgSet( dlg, IDC_EDIT_mes, mesfile(1)) 
!		endif
!
!      end select
!
!      retdir = changedirqq (actdir)
!
!END SUBROUTINE BrowseFile
!


!***************************************************************


SUBROUTINE GetFile ( filename, filter_spec, text )
!SUBROUTINE GetFile ( dlg, id, callbacktype )

use dflogm

implicit none
!type (dialog) dlg
integer id
!integer callbacktype
!integer retint
logical ex
character(512):: filename 
character(*)  :: text

!Settings for FileOpen
character*512 :: file_get 
character*(*) :: filter_spec 
!character*(*),parameter :: filter_spec = &
!  "Simulation control Files (*.sim)"C//"*.sim"C// &
!  "All Files (*.*)"C//"*.*"C//""C

include '4C_dialogs.fd'

logical retlog

         call FileOpen (filename, filter_spec)
         inquire (File = filename, exist = ex)
         if(ex .eqv. .true.) then
            write(*,'(A,A)') ' Simulation control file: ', trim(filename)
         else
			filename = ' '
         endif

END SUBROUTINE GetFile

!***************************************************************


! needed for time_out in Open Existing File
SUBROUTINE GetFileF ( filename, filter_spec, text )
!SUBROUTINE GetFile ( dlg, id, callbacktype )

use dflogm
!use dflib

implicit none

!type (dialog) dlg
integer id
!integer callbacktype
!integer retint
logical ex
character(150) :: filename 
character(*) :: text

!Settings for FileOpen
character*512 :: file_get 
character*(*) :: filter_spec 
!character*(*),parameter :: filter_spec = &
!  "Simulation control Files (*.sim)"C//"*.sim"C// &
!  "All Files (*.*)"C//"*.*"C//""C

include '4C_dialogs.fd'

logical retlog
!logical retdir

        call FileOpen (file_get, filter_spec)
        do       
!         filename = trim(file_get)
         filename = file_get
!         inquire (File = filename, exist = ex)
        ex = .true.
        if(ex .eqv. .true.) then
            write(*,'(A,A)') ' Simulation control file: ', trim(filename)
            exit
         else
            write(*,'(A)') ' Simulation control file: ', trim(filename), ' not exists'
            write(*,'(A)',advance='no') ' Try again' 
            PAUSE
            call FileOpen (file_get, filter_spec)
            cycle
         endif
        enddo 

!   retdir = changedirqq (actdir)
       
END SUBROUTINE GetFileF


!!***************************************************************
!
!SUBROUTINE SetFlag ( dlg, id, nflag, actflag, charflag )
!
!use dflogm
!
!use data_simul
!
!implicit none
!
!type (dialog) dlg
!integer id
!integer nflag, actflag     ! number of options; counter; actual option
!character(60), dimension(30) :: charflag
!character(4) text, helpsim
!integer i, ichar
!
!include '4C_dialogs.fd'
!
!logical retlog
!
!
!	! set options
!	retlog = DlgSet( dlg, id, nflag) 
!	do i = 1,nflag    
!		retlog = DlgSet( dlg, id, charflag(i),i) 
!	enddo
!	retlog = DlgSet( dlg, id, trim(charflag(actflag))) 
!
!END SUBROUTINE SetFlag
!
!!***************************************************************
!
!SUBROUTINE SetFlag100 ( dlg, id, nflag, actflag, charflag )
!
!use dflogm
!
!use data_simul
!
!implicit none
!
!type (dialog) dlg
!integer id
!integer nflag, actflag     ! number of options; counter; actual option
!character(100), dimension(30) :: charflag
!character(4) text, helpsim
!integer i, ichar
!
!include '4C_dialogs.fd'
!
!logical retlog
!
!
!	! set options
!	retlog = DlgSet( dlg, id, nflag) 
!	do i = 1,nflag    
!		retlog = DlgSet( dlg, id, charflag(i),i) 
!	enddo
!	retlog = DlgSet( dlg, id, trim(charflag(actflag))) 
!
!END SUBROUTINE SetFlag100
!
!!***************************************************************
!
!! sets the flag value from the flags dlg
!subroutine GetFlag (dlg, id, actflag)
!
!use dflogm
!
!use flag_field
!
!implicit none
!
!type (dialog) dlg
!character(150) text
!integer actflag, id, ipos
!logical retlog
!
!include '4C_dialogs.fd'
!
!	retlog = DLGGET (dlg, id, text)
!	text = adjustl(text)      ! remove leading blanks
!	ipos = index(text, ' ')
!	text = text(1:ipos)
!	read (text, *) flagsave(actflag, act_run)
!
!end subroutine
!
!!***************************************************************
!
!SUBROUTINE SetCheckOut(dlg, id, outa, inum)
!
!! Set output-file (Check-Box) 
!use dflogm
!USE DFLIB
!
!use data_out
!use data_simul
!
!implicit none
!
!type (dialog) dlg
!integer id
!!integer callbacktype
!integer inum
!character(10) text1
!character(100) text2
!logical check_state
!logical retlog
!type (out_struct),dimension(outy_n),target :: outa   !  information about output files
!
!include '4C_dialogs.fd'
!
!    if (outa(inum)%out_flag .gt. 0) then
!       check_state = .TRUE.
!    else
!       check_state = .FALSE.
!    endif
!
!    text1   = adjustl(outa(inum)%kind_name)
!    text2   = text1//' - '//outa(inum)%f_line(3:)
!    retlog = DlgSet(dlg, id, text2) 
!!    retlog = DlgSet(dlg, id, outa(inum)%kind_name//' - '//outa(inum)%f_line(3:)) 
!    retlog = DlgSet(dlg, id, check_state) 
!
!
!END SUBROUTINE SetCheckOut
